/*
 * Copyright 2018, CS Systemes d'Information, http://www.c-s.fr
 *
 * This file is part of EODAG project
 *     https://www.github.com/CS-SI/EODAG
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
syntax = "proto3";

package eodag;

// a protobuf message for numpy ndarray, inspired from the gist at
// https://gist.github.com/monkeybutter/b91004077be5d73a478a and comments from
// https://github.com/google/protobuf/issues/880
message NDArray {
    repeated double array = 1 [packed=true];
    repeated fixed32 shape = 2 [packed=true];
    string dtype = 3;
}

message EOProductSubdataset {
    string id = 1;
    string producer = 2;
    string product_type = 3;
    string platform = 4;
    string sensor = 5;

    NDArray data = 6;
}